package com.twofours.surespot.network;

import android.content.Context;
import android.content.Intent;
import android.util.Log;

import com.google.android.gcm.GCMRegistrar;
import com.loopj.android.http.AsyncHttpClient;
import com.loopj.android.http.AsyncHttpResponseHandler;
import com.loopj.android.http.JsonHttpResponseHandler;
import com.loopj.android.http.PersistentCookieStore;
import com.loopj.android.http.RequestParams;
import com.loopj.android.http.ResponseHandlerInterface;
import com.loopj.android.http.SyncHttpClient;
import com.twofours.surespot.SurespotApplication;
import com.twofours.surespot.SurespotConstants;
import com.twofours.surespot.Utils;
import com.twofours.surespot.ui.activities.LoginActivity;

import org.apache.http.HttpException;
import org.apache.http.HttpResponse;
import org.apache.http.HttpResponseInterceptor;
import org.apache.http.HttpStatus;
import org.apache.http.client.CookieStore;
import org.apache.http.cookie.Cookie;
import org.apache.http.impl.client.DefaultHttpClient;
import org.apache.http.protocol.HttpContext;

import java.io.ByteArrayInputStream;
import java.io.IOException;
import java.net.URI;
import java.util.HashMap;
import java.util.Map;

import cz.msebera.android.httpclient.Header;

public class NetworkController {
	protected static final String TAG = "NetworkController";
	private static Cookie mConnectCookie;

	private static synchronized void setConnectCookie(Cookie connectCookie) {
		// we be authorized
		NetworkController.mConnectCookie = connectCookie;
		setUnauthorized(false);
	}

	private static AsyncHttpClient mClient;
	private static cz.msebera.android.httpclient.client.CookieStore mCookieStore;
	private static SyncHttpClient mSyncClient;

	public static void get(String url, RequestParams params, AsyncHttpResponseHandler responseHandler) {
		mClient.get(SurespotConstants.BASE_URL + url, params, responseHandler);
	}

	public static void post(String url, RequestParams params, AsyncHttpResponseHandler responseHandler) {
		mClient.post(SurespotConstants.BASE_URL + url, params, responseHandler);
	}

	public static Cookie getConnectCookie() {
		return mConnectCookie;
	}

	public static boolean hasSession() {
		return mConnectCookie != null;
	}

	public static cz.msebera.android.httpclient.client.CookieStore getCookieStore() {
		return mCookieStore;
	}

	private static boolean mUnauthorized;

	private static boolean isUnauthorized() {
		return mUnauthorized;
	}

	public static synchronized void setUnauthorized(boolean unauthorized) {

		NetworkController.mUnauthorized = unauthorized;
	}

	static {
		mCookieStore = new PersistentCookieStore(SurespotApplication.getAppContext());
		if (mCookieStore.getCookies().size() > 0) {
			Log.v(TAG, "mmm cookies in the jar: " + mCookieStore.getCookies().size());
			mConnectCookie = extractConnectCookie((CookieStore) mCookieStore);
		}

		mClient = new AsyncHttpClient();
		
		mSyncClient = new SyncHttpClient();

		HttpResponseInterceptor httpResponseInterceptor = new HttpResponseInterceptor() {

			@Override
			public void process(HttpResponse response, HttpContext context) throws HttpException, IOException {

				if (response.getStatusLine().getStatusCode() == HttpStatus.SC_UNAUTHORIZED) {
					String origin = context.getAttribute("http.cookie-origin").toString();

					if (origin != null) {

						if (!NetworkController.isUnauthorized()) {

							if (!(origin.contains(SurespotConstants.BASE_URL.substring(7)) && origin.contains("/login"))) {

								mClient.cancelRequests(SurespotApplication.getAppContext(), true);

								Log.v(TAG, "Got 401, launching login intent.");
								Intent intent = new Intent(SurespotApplication.getAppContext(), LoginActivity.class);
								intent.addFlags(Intent.FLAG_ACTIVITY_NEW_TASK);
								SurespotApplication.getAppContext().startActivity(intent);

								setUnauthorized(true);
							}

						}
					}
				}
			}
		};

		mClient.setCookieStore(mCookieStore);
		mSyncClient.setCookieStore(mCookieStore);
		// handle 401s
		((DefaultHttpClient) mClient.getHttpClient()).addResponseInterceptor(httpResponseInterceptor);
		((DefaultHttpClient) mSyncClient.getHttpClient()).addResponseInterceptor(httpResponseInterceptor);

//		enableHttpResponseCache();
	}
	
//	private static void enableHttpResponseCache() {
//	    try {
//	        long httpCacheSize = 10 * 1024 * 1024; // 10 MiB
//	        File httpCacheDir = new File(SurespotApplication.getAppContext().getCacheDir(), "http");
//	        Class.forName("android.net.http.HttpResponseCache")
//	            .getMethod("install", File.class, long.class)
//	            .invoke(null, httpCacheDir, httpCacheSize);
//	    } catch (Exception httpResponseCacheNotAvailable) {
//	    }
//	}

	public static void addUser(String username, String password, String publicKey, final AsyncHttpResponseHandler responseHandler) {
		Map<String, String> params = new HashMap<String, String>();
		params.put("username", username);
		params.put("password", password);
		params.put("publickey", publicKey);
		// get the gcm id
		final String gcmIdReceived = Utils.getSharedPrefsString(SurespotConstants.PrefNames.GCM_ID_RECEIVED);

		boolean gcmUpdatedTemp = false;
		// update the gcmid if it differs
		if (gcmIdReceived != null) {

			params.put("gcmId", gcmIdReceived);
			gcmUpdatedTemp = true;
		}

		// just be javascript already
		final boolean gcmUpdated = gcmUpdatedTemp;

		post("/users", new RequestParams(params), new AsyncHttpResponseHandler() {

			@Override
			public void onSuccess(int responseCode, Header[] headers, byte[] result) {
				setConnectCookie(extractConnectCookie((CookieStore) mCookieStore));
				if (mConnectCookie == null) {
					Log.e(TAG, "did not get cookie from signup");
					responseHandler.onFailure(responseCode,headers, "Did not get cookie.".getBytes(),new Exception("Did not get cookie."));
				} else {
					// update shared prefs
					if (gcmUpdated) {
						Utils.putSharedPrefsString(SurespotConstants.PrefNames.GCM_ID_SENT, gcmIdReceived);
					}

					responseHandler.onSuccess(responseCode, headers,result);
				}

			}

			@Override
			public void onFailure(int code, Header[] headers, byte[] content, Throwable arg0) {
				responseHandler.onFailure(code,headers,content,arg0);
			}

			@Override
			public void onFinish() {
				responseHandler.onFinish();
			}

		});

	}

	private static Cookie extractConnectCookie(CookieStore cookieStore) {
		for (Cookie c : cookieStore.getCookies()) {
			// System.out.println("Cookie name: " + c.getName() + " value: " +
			// c.getValue());
			if (c.getName().equals("connect.sid")) {
				return c;
			}
		}
		return null;

	}

	public static void login(String username, String password, final AsyncHttpResponseHandler responseHandler) {
		Map<String, String> params = new HashMap<String, String>();
		params.put("username", username);
		params.put("password", password);

		// get the gcm id
		final String gcmIdReceived = Utils.getSharedPrefsString(SurespotConstants.PrefNames.GCM_ID_RECEIVED);
		String gcmIdSent = Utils.getSharedPrefsString(SurespotConstants.PrefNames.GCM_ID_SENT);

		boolean gcmUpdatedTemp = false;
		// update the gcmid if it differs
		if (gcmIdReceived != null && !gcmIdReceived.equals(gcmIdSent)) {

			params.put("gcmId", gcmIdReceived);
			gcmUpdatedTemp = true;
		}

		// just be javascript already
		final boolean gcmUpdated = gcmUpdatedTemp;

		post("/login", new RequestParams(params), new AsyncHttpResponseHandler() {

			@Override
			public void onSuccess(int responseCode, Header[] headers, byte[] result) {
				setConnectCookie(extractConnectCookie((CookieStore) mCookieStore));
				if (mConnectCookie == null) {
					Log.e(TAG, "Did not get cookie from login.");
					responseHandler.onFailure(responseCode,headers,null,new Exception("Did not get cookie."));
				} else {
					// update shared prefs
					if (gcmUpdated) {
						Utils.putSharedPrefsString(SurespotConstants.PrefNames.GCM_ID_SENT, gcmIdReceived);
					}

					responseHandler.onSuccess(responseCode, headers,result);
				}

			}

			@Override
			public void onFailure(int code, Header[] headers, byte[] content, Throwable arg0) {
				responseHandler.onFailure(code,headers,content,arg0);
			}

			@Override
			public void onFinish() {
				responseHandler.onFinish();
			}
		});

	}

	public static void getFriends(AsyncHttpResponseHandler responseHandler) {
		get("/friends", null, responseHandler);
	}

	// if we have an id get the messages since the id, otherwise get the last x
	public static void getMessages(String room, String id, AsyncHttpResponseHandler responseHandler) {

		if (id == null) {
			get("/messages/" + room, null, responseHandler);
		} else {
			get("/messages/" + room + "/after/" + id, null, responseHandler);
		}
	}

	// if we have an id get the messages since the id, otherwise get the last x
	public static void getEarlierMessages(String room, String id, AsyncHttpResponseHandler responseHandler) {
		get("/messages/" + room + "/before/" + id, null, responseHandler);
	}

	public static void getLastMessageIds(JsonHttpResponseHandler responseHandler) {
		get("/conversations/ids", null, responseHandler);
	}

	public static void getPublicKey(String username, AsyncHttpResponseHandler responseHandler) {
		get("/publickey/" + username, null, responseHandler);

	}

	public static String getPublicKeySync(String username) {
		return mSyncClient.get(SurespotConstants.BASE_URL + "/publickey/" + username, new ResponseHandlerInterface() {
			@Override
			public void sendResponseMessage(cz.msebera.android.httpclient.HttpResponse response) throws IOException {

			}

			@Override
			public void sendStartMessage() {

			}

			@Override
			public void sendFinishMessage() {

			}

			@Override
			public void sendProgressMessage(long bytesWritten, long bytesTotal) {

			}

			@Override
			public void sendCancelMessage() {

			}

			@Override
			public void sendSuccessMessage(int statusCode, Header[] headers, byte[] responseBody) {

			}

			@Override
			public void sendFailureMessage(int statusCode, Header[] headers, byte[] responseBody, Throwable error) {

			}

			@Override
			public void sendRetryMessage(int retryNo) {

			}

			@Override
			public URI getRequestURI() {
				return null;
			}

			@Override
			public void setRequestURI(URI requestURI) {

			}

			@Override
			public Header[] getRequestHeaders() {
				return new Header[0];
			}

			@Override
			public void setRequestHeaders(Header[] requestHeaders) {

			}

			@Override
			public boolean getUseSynchronousMode() {
				return false;
			}

			@Override
			public void setUseSynchronousMode(boolean useSynchronousMode) {

			}

			@Override
			public boolean getUsePoolThread() {
				return false;
			}

			@Override
			public void setUsePoolThread(boolean usePoolThread) {

			}

			@Override
			public void onPreProcessResponse(ResponseHandlerInterface instance, cz.msebera.android.httpclient.HttpResponse response) {

			}

			@Override
			public void onPostProcessResponse(ResponseHandlerInterface instance, cz.msebera.android.httpclient.HttpResponse response) {

			}

			@Override
			public Object getTag() {
				return null;
			}

			@Override
			public void setTag(Object TAG) {

			}
		}).toString();
	}

	public static void invite(String friendname, AsyncHttpResponseHandler responseHandler) {

		post("/invite/" + friendname, null, responseHandler);

	}

	public static void respondToInvite(String friendname, String action, AsyncHttpResponseHandler responseHandler) {
		post("/invites/" + friendname + "/" + action, null, responseHandler);
	}

	public static void registerGcmId(final AsyncHttpResponseHandler responseHandler) {
		// make sure the gcm is set
		// use case:
		// user signs-up without google account (unlikely)
		// user creates google account
		// user opens app again, we have session so neither login or add user is called (which wolud set the gcm)
		// so we need to upload the gcm here if we haven't already
		// get the gcm id

		final String gcmIdReceived = Utils.getSharedPrefsString(SurespotConstants.PrefNames.GCM_ID_RECEIVED);
		String gcmIdSent = Utils.getSharedPrefsString(SurespotConstants.PrefNames.GCM_ID_SENT);

		Map<String, String> params = new HashMap<String, String>();

		boolean gcmUpdatedTemp = false;
		// update the gcmid if it differs
		if (gcmIdReceived != null && !gcmIdReceived.equals(gcmIdSent)) {

			params.put("gcmId", gcmIdReceived);
			gcmUpdatedTemp = true;
		} else {
			Log.v(TAG, "GCM does not need updating on server.");
			return;
		}

		// just be javascript already
		final boolean gcmUpdated = gcmUpdatedTemp;

		post("/registergcm", new RequestParams(params), new AsyncHttpResponseHandler() {

			@Override
			public void onSuccess(int responseCode, Header[] headers, byte[] result) {

				// update shared prefs
				if (gcmUpdated) {
					Utils.putSharedPrefsString(SurespotConstants.PrefNames.GCM_ID_SENT, gcmIdReceived);
				}

				responseHandler.onSuccess(responseCode, headers,result);
			}

			@Override
			public void onFailure(int code, Header[] headers, byte[] arg1, Throwable arg0) {
				responseHandler.onFailure(code,headers, arg1,arg0);
			}

		});

	}

	public static void userExists(String username, AsyncHttpResponseHandler responseHandler) {
		get("/users/" + username + "/exists", null, responseHandler);
	}

	/**
	 * Unregister this account/device pair within the server.
	 */
	public static void unregister(final Context context, final String regId) {
		Log.i(TAG, "unregistering device (regId = " + regId + ")");
		GCMRegistrar.setRegisteredOnServer(context, false);
	}

	public static void postFile(Context context, String user, String id, byte[] data, String mimeType,
			AsyncHttpResponseHandler responseHandler) {

		RequestParams params = new RequestParams();
		params.put("image", new ByteArrayInputStream(data), id, mimeType);

		post("/images/" + user, params, responseHandler);

	}

	public static void getFile(String user, String id, AsyncHttpResponseHandler responseHandler) {
		get("/images/" + user + "/" + id, null, responseHandler);
	}

}
